/**
 * Performs right-to-left function composition.
 * @signature compose :: [(m -> n), ..., (b -> c), (a -> b)] -> a -> n
 * @func
 * @example
 * const addOne = x => x + 1;
 * const timeTen = x => x * 10;
 * const addOneTimeTen = compose(timeTen, addOne);
 * const result = addOneTimeTen(9); // 100
 */
var compose = function compose() {
  for (var _len = arguments.length, fns = Array(_len), _key = 0; _key < _len; _key++) {
    fns[_key] = arguments[_key];
  }

  return fns.reduce(function (f, g) {
    return function () {
      return f(g.apply(undefined, arguments));
    };
  });
};

export default compose;