/**
 * Wraps the given function, if the number of provided args is sufficient, call the passed function fn.
 * Otherwise, return a wrapped function fn that expects the rest of the arguments.
 * If you want to curry a function that accepts a variable number of arguments (a variadic function),
 * you can optionally pass the number of arguments to the second parameter arity.
 * @signature curry :: ((a, b, ..., n) -> m) -> a -> b -> ...n -> m
 * @func
 * @example
 * const add = curry((x, y) => x + y);
 * const addFiveTo = add(5);
 * addFiveTo(10); // 15
 */
function curry(fn) {
  var arity = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : fn.length;

  for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  return arity <= args.length ? fn.apply(undefined, args) : curry.bind.apply(curry, [null, fn, arity].concat(args));
}

export default curry;